// --------------------------------------------------------------
// $Id: $
/** @version 1.0 07/22/97
 * @author P. Thyagarajan
 * copyright 1996-1997 InfoPlace Inc.  All Rights Reserved.
 *
 * Class UsersSample
 */
// --------------------------------------------------------------
// current package ...
// 

// --------------------------------------------------------------
// imports ...
import com.infoplace.simpledb.*;

import java.util.Vector;

// --------------------------------------------------------------
/** This is a sample program showing off the database. 
 *  The program creates a database with two tables, Users and
 *  Groups. Also, the program populates both the tables with
 *  some records.
 *  <P>
 *  The program can be execute as:
 *   > java UsersSample -create         // to create the database and tables
 *   > java UsersSample -list user all  // to list all the users in the database
 *   > java UsersSample -list group all // to list all the groups in the database
 *   > java UsersSample -list group <id>// to list a group identified by group name <id>
 *   > java UsersSample -list user <id> // to list a user identified by userid <id>
 */
// --------------------------------------------------------------
class UsersSample {

  String    fDBName;
  String    fDBPathName;
  DB        fDatabase;
  DBTable   fGroupTable;
  DBTable   fUserTable;

public static void main(String argv[]) {
  UsersSample o = new UsersSample();
  o.doIt(argv);
}

public void doIt(String argv[]) {

  fDBName = "UserDB";
  fDBPathName = ".";

  int i = 0;
  boolean inputError = argv.length < 1;
  boolean isCreate = false;
  boolean isGroup = false;
  String value = "";

  if (!inputError) {
    if (argv[i].equals("-create"))
      isCreate = true;
    else if (argv[i].equals("-list"))
      isCreate = false;
    else
      inputError = true;
  }
  if (!isCreate) {
    inputError = argv.length != 3;
    if (!inputError) {
      i++;
      if (argv[i].equals("group"))
	isGroup = true;
      else if (argv[i].equals("user"))
	isGroup = false;
      else 
	inputError = true;

      i++;
      value = argv[i];
    }
  }
  else {
    inputError = argv.length != 1;
  }
  if (inputError) {
    System.out.println("Usage: [-create] | [-list [group | user] [all | <id>]]");
    System.exit(-2);
  }
  if (isCreate) {
    createDB();
    createTables();
    populateGroups();
    populateUsers();
    System.out.println("Database created.");
  }
  else {
    openDB();
    openTables();
    if (isGroup) {
      if (value.equals("all"))
	listAllGroups();
      else 
	printGroup(value);
    }
    else {
      if (value.equals("all"))
	listAllUsers();
      else 
	printUser(value);
    }
  }
  System.exit(0);
}

public void createDB() {
  fDatabase = null;
  try {
    if (DB.exists(fDBName, fDBPathName))  
      DB.delete(fDBName, fDBPathName);
    fDatabase = new DB(fDBName, fDBPathName);
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }
}

public void openDB() {
  fDatabase = null;
  try {
    fDatabase = new DB(fDBName, fDBPathName, DB.kReadWriteMode);
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }
}

public void createTables() {
  try {
    String[] columns1 = {"Name", "Members"};
    String[] types1 = {DBTable.kStringType, DBTable.kListOfStringsType};
    fGroupTable = fDatabase.createTable("Groups", columns1, types1, 500);

    String[] columns2 = {"UserID", "Name", "Group", "Age"};
    String[] types2 = {DBTable.kStringType, DBTable.kStringType, DBTable.kStringType, DBTable.kNumberType};
    fUserTable = fDatabase.createTable("Users", columns2, types2, 5000);    
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }
}

public void openTables() {
  try {
    fGroupTable = fDatabase.openTable("Groups");
    fUserTable = fDatabase.openTable("Users");
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }
}

public void populateGroups() {
  try {
    String[] groups = {"Red", "Green", "Blue", "White", "Black"};
    Object[] record = new Object[fGroupTable.getColumnCount()];
    for (int i = 0; i < groups.length; i++) {
      record[0] = groups[i];
      fGroupTable.insertRecord(record);
    }
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }
}

public void populateUsers() {
  try {
    String[] userids = {"orange", "apple", "banana", "mango", "pear", "grapes"};
    String[] names = {"Jack A", "Jack B", "jack C", "Jack D", "Jack E", "Jack F"};
    String[] groups = {"Red", "Red", "Blue", "White", "White", "Green"};
    int[] ages = {10, 20, 30, 40, 50, 60};
    Object[] record = new Object[fUserTable.getColumnCount()];
    Object[] groupRecord = new Object[fGroupTable.getColumnCount()];
    
    for (int i = 0; i < groups.length; i++) {
      record[0] = userids[i];
      record[1] = names[i];
      record[2] = groups[i];
      record[3] = new Integer(ages[i]);
      fUserTable.insertRecord(record);
      fGroupTable.retreiveRecord(groups[i], groupRecord);
      groupRecord[1] = fUserTable.appendToListValue(groupRecord[1], userids[i]);
      fGroupTable.updateRecord(groups[i], groupRecord);
    }
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }
}

public void listAllGroups() {
  try {
    DBIterator iterator = fGroupTable.createIterator();
    fGroupTable.print(System.out, " | ", iterator);
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }  
}

public void listAllUsers() {
  try {
    DBIterator iterator = fUserTable.createIterator();
    fUserTable.print(System.out, " | ", iterator);
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }  
}

public void printGroup(String name) {
  try {
    Object[] record = new Object[fGroupTable.getColumnCount()];
    fGroupTable.retreiveRecord(name, record);
    fGroupTable.print(System.out, " | ", record);
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }  
}

public void printUser(String userid) {
  try {
    Object[] record = new Object[fUserTable.getColumnCount()];
    fUserTable.retreiveRecord(userid, record);
    fUserTable.print(System.out, " | ", record);
  }
  catch (DBException e) {
    System.out.println(e.getMessage());
    System.exit(-1);
  }  
}
}
